@@ -21,7 +21,7 @@ function p = bspeval(d,c,k,u)
 %
 %    This program is free software: you can redistribute it and/or modify
 %    it under the terms of the GNU General Public License as published by
-%    the Free Software Foundation, either version 2 of the License, or
+%    the Free Software Foundation, either version 3 of the License, or
 %    (at your option) any later version.
 
 %    This program is distributed in the hope that it will be useful,
@@ -43,7 +43,7 @@ nu = numel(u);
                                                 %   double **ctrl = vec2mat(c,mc,nc);
                                                 %
                                                 %   // Contruct the evaluated points
-p = zeros(mc,nu);                               %   double **pnt = vec2mat(p,mc,nu);
+                                                %   double **pnt = vec2mat(p,mc,nu);
                                                 %
                                                 %   // space for the basis functions
 %N = zeros(d+1,1);                               %   double *N = (double*) mxMalloc((d+1)*sizeof(double));
