@@ -1,15 +1,18 @@
-function nrbkntplot (nurbs)
+function nrbkntplot (nurbs, nsub)
 
 % NRBKNTPLOT: Plot a NURBS entity with the knots subdivision.
 % 
 % Calling Sequence:
 % 
 %   nrbkntplot(nurbs)
+%   nrbkntplot(nurbs, npnts)
 % 
 % INPUT:
 % 
 %   nurbs: NURBS curve, surface or volume, see nrbmak.
-% 
+%   npnts: Number of evaluation points, for a surface or volume, a row 
+%       vector with the number of points along each direction.
+%
 % Example:
 %
 %   Plot the test surface with its knot vector
@@ -20,11 +23,11 @@ function nrbkntplot (nurbs)
 % 
 %   nrbctrlplot
 %
-%    Copyright (C) 2011 Rafael Vazquez
+%    Copyright (C) 2011, 2012, 2021 Rafael Vazquez
 %
 %    This program is free software: you can redistribute it and/or modify
 %    it under the terms of the GNU General Public License as published by
-%    the Free Software Foundation, either version 2 of the License, or
+%    the Free Software Foundation, either version 3 of the License, or
 %    (at your option) any later version.
 
 %    This program is distributed in the hope that it will be useful,
@@ -49,90 +52,70 @@ hold_flag = ishold;
 
 if (iscell (nurbs.knots))
  if (size (nurbs.knots,2) == 2) % plot a NURBS surface
-   nsub = 100;
-   nrbplot (nurbs, [nsub nsub], 'light', light, 'colormap', cmap);
+   if (nargin < 2)
+     nsub = [50 50];
+   elseif (numel(nsub) == 1)
+     nsub = [nsub nsub];
+   end
+   nrbplot (nurbs, nsub, 'light', light, 'colormap', cmap);
    hold on
 
    % And plot the knots
-   knt1 = unique (nurbs.knots{1});
-   knt2 = unique (nurbs.knots{2});
-   p1 = nrbeval (nurbs, {knt1, linspace(0.0,1.0,nsub)});
-   p2 = nrbeval (nurbs, {linspace(0.0,1.0,nsub), knt2});
+   knt1 = unique (nurbs.knots{1}(nurbs.order(1):end-nurbs.order(1)+1));
+   knt2 = unique (nurbs.knots{2}(nurbs.order(2):end-nurbs.order(2)+1));
+   p1 = nrbeval (nurbs, {knt1, linspace(knt2(1),knt2(end),nsub(2)+1)});
+   p2 = nrbeval (nurbs, {linspace(knt1(1),knt1(end),nsub(1)+1), knt2});
 
   if (any (nurbs.coefs(3,:)))
     % surface in a 3D space
     for ii = 1:numel(knt1)
-      plot3 (squeeze(p1(1,ii,:)), squeeze(p1(2,ii,:)), squeeze(p1(3,ii,:)));
+      plot3 (squeeze(p1(1,ii,:)), squeeze(p1(2,ii,:)), squeeze(p1(3,ii,:)),'k');
     end
     for ii = 1:numel(knt2)
-      plot3 (squeeze(p2(1,:,ii)), squeeze(p2(2,:,ii)), squeeze(p2(3,:,ii))); 
+      plot3 (squeeze(p2(1,:,ii)), squeeze(p2(2,:,ii)), squeeze(p2(3,:,ii)),'k'); 
     end
   else
     % plain surface
     for ii = 1:numel(knt1)
-      plot (squeeze(p1(1,ii,:)), squeeze (p1(2,ii,:))); 
+      plot (squeeze(p1(1,ii,:)), squeeze (p1(2,ii,:)),'k'); 
     end
     for ii = 1:numel(knt2)
-      plot (p2(1,:,ii),p2(2,:,ii));
+      plot (p2(1,:,ii),p2(2,:,ii),'k');
     end
   end
 
 
 
  elseif (size (nurbs.knots,2) == 3) % plot a NURBS volume
-   nsub = 30;
-   nrbplot (nurbs, [nsub nsub nsub], 'light', light, 'colormap', cmap);
+   if (nargin < 2)
+     nsub = [25 25 25];
+   elseif (numel(nsub) == 1)
+     nsub = [nsub nsub nsub];
+   end
+   % Plot the boundaries
+   bnd = nrbextract (nurbs);
+   nrbkntplot (bnd(1), nsub(2:3));
    hold on
-
-   % And plot the knots
-   knt1 = unique (nurbs.knots{1});
-   knt2 = unique (nurbs.knots{2});
-   knt3 = unique (nurbs.knots{3});
-   kv_face1 = nrbeval (nurbs, {0, knt2, linspace(0.0,1.0,nsub)});
-   kw_face1 = nrbeval (nurbs, {0, linspace(0.0,1.0,nsub), knt3});
-   kv_face2 = nrbeval (nurbs, {1, knt2, linspace(0.0,1.0,nsub)});
-   kw_face2 = nrbeval (nurbs, {1, linspace(0.0,1.0,nsub), knt3});
-   ku_face3 = nrbeval (nurbs, {knt1, 0, linspace(0.0,1.0,nsub)});
-   kw_face3 = nrbeval (nurbs, {linspace(0.0,1.0,nsub), 0, knt3});
-   ku_face4 = nrbeval (nurbs, {knt1, 1, linspace(0.0,1.0,nsub)});
-   kw_face4 = nrbeval (nurbs, {linspace(0.0,1.0,nsub), 1, knt3});
-   ku_face5 = nrbeval (nurbs, {knt1, linspace(0.0,1.0,nsub), 0});
-   kv_face5 = nrbeval (nurbs, {linspace(0.0,1.0,nsub), knt2, 0});
-   ku_face6 = nrbeval (nurbs, {knt1, linspace(0.0,1.0,nsub), 1});
-   kv_face6 = nrbeval (nurbs, {linspace(0.0,1.0,nsub), knt2, 1});
-
-   for ii = 1:numel(knt1)
-     plot3 (squeeze (ku_face3(1,ii,:,:)), squeeze (ku_face3(2,ii,:,:)), squeeze (ku_face3(3,ii,:,:))); 
-     plot3 (squeeze (ku_face4(1,ii,:,:)), squeeze (ku_face4(2,ii,:,:)), squeeze (ku_face4(3,ii,:,:))); 
-     plot3 (squeeze (ku_face5(1,ii,:,:)), squeeze (ku_face5(2,ii,:,:)), squeeze (ku_face5(3,ii,:,:))); 
-     plot3 (squeeze (ku_face6(1,ii,:,:)), squeeze (ku_face6(2,ii,:,:)), squeeze (ku_face6(3,ii,:,:))); 
-   end
-   for ii = 1:numel(knt2)
-     plot3 (squeeze (kv_face1(1,:,ii,:)), squeeze (kv_face1(2,:,ii,:)), squeeze (kv_face1(3,:,ii,:))); 
-     plot3 (squeeze (kv_face2(1,:,ii,:)), squeeze (kv_face2(2,:,ii,:)), squeeze (kv_face2(3,:,ii,:))); 
-     plot3 (squeeze (kv_face5(1,:,ii,:)), squeeze (kv_face5(2,:,ii,:)), squeeze (kv_face5(3,:,ii,:))); 
-     plot3 (squeeze (kv_face6(1,:,ii,:)), squeeze (kv_face6(2,:,ii,:)), squeeze (kv_face6(3,:,ii,:))); 
-   end
-   for ii = 1:numel(knt3)
-     plot3 (squeeze (kw_face1(1,:,:,ii)), squeeze(kw_face1(2,:,:,ii)), squeeze (kw_face1(3,:,:,ii))); 
-     plot3 (squeeze (kw_face2(1,:,:,ii)), squeeze(kw_face2(2,:,:,ii)), squeeze (kw_face2(3,:,:,ii))); 
-     plot3 (squeeze (kw_face3(1,:,:,ii)), squeeze(kw_face3(2,:,:,ii)), squeeze (kw_face3(3,:,:,ii))); 
-     plot3 (squeeze (kw_face4(1,:,:,ii)), squeeze(kw_face4(2,:,:,ii)), squeeze (kw_face4(3,:,:,ii))); 
+   for iface = 2:6
+     inds = setdiff(1:3, ceil(iface/2));
+     nrbkntplot (bnd(iface), nsub(inds));
    end
  end
-
 else % plot a NURBS curve
-  nsub = 1000;
+  if (nargin < 2)
+    nsub = 1000;
+  end
   nrbplot (nurbs, nsub);
   hold on
 
   % And plot the knots
-   p = nrbeval (nurbs, unique (nurbs.knots));
+   order = nurbs.order;
+   p = nrbeval (nurbs, unique (nurbs.knots(order:end-order+1)));
 
    if (any (nurbs.coefs(3,:))) % plot a 3D curve
-     plot3 (p(1,:), p(2,:), p(3,:), 'x'); 
+     plot3 (p(1,:), p(2,:), p(3,:), 'rx'); 
    else                     % plot a 2D curve
-     plot (p(1,:), p(2,:), 'x'); 
+     plot (p(1,:), p(2,:), 'rx'); 
    end
 
 end
